# Generated by Django 5.0 on 2023-12-18 20:23

import dcat.models
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Agent",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("type", models.CharField(max_length=20)),
                ("mbox", models.EmailField(blank=True, max_length=254, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Checksum",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("checksum_value", models.CharField(max_length=255)),
                ("algorithm", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="DataTheme",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("code", models.CharField(max_length=10)),
                ("label", models.CharField(max_length=255)),
                ("description", models.TextField(blank=True, default="")),
            ],
        ),
        migrations.CreateModel(
            name="LicenceDocument",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=255)),
                (
                    "code",
                    models.CharField(blank=True, max_length=10, null=True, unique=True),
                ),
                ("url_general", models.URLField(blank=True, default="")),
                ("url_document", models.URLField(blank=True, default="")),
            ],
        ),
        migrations.CreateModel(
            name="MediaType",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("extension", models.CharField(max_length=10)),
                (
                    "code",
                    models.CharField(blank=True, max_length=10, null=True, unique=True),
                ),
                ("media_type", models.CharField(blank=True, max_length=50)),
                ("description", models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name="Catalog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("description", models.TextField()),
                (
                    "publisher",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="dcat.agent"
                    ),
                ),
                ("themes", models.ManyToManyField(to="dcat.datatheme")),
                (
                    "license",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dcat.licencedocument",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Dataset",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("description", models.TextField()),
                (
                    "catalog",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="dcat.catalog"
                    ),
                ),
                (
                    "publisher",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dcat.agent",
                    ),
                ),
                ("themes", models.ManyToManyField(to="dcat.datatheme")),
            ],
        ),
        migrations.CreateModel(
            name="Distribution",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(blank=True, max_length=255)),
                ("description", models.TextField(blank=True)),
                (
                    "file",
                    models.FileField(
                        upload_to=dcat.models.Distribution._get_storage_path
                    ),
                ),
                ("external_download_url", models.URLField(blank=True, default="")),
                (
                    "checksum",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dcat.checksum",
                    ),
                ),
                (
                    "dataset",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="dcat.dataset"
                    ),
                ),
                (
                    "license",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dcat.licencedocument",
                    ),
                ),
                (
                    "format",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dcat.mediatype",
                    ),
                ),
            ],
        ),
    ]



import os
import xml.etree.ElementTree as ET

from django.core.management.base import BaseCommand
from dcat.models import LicenceDocument


class Command(BaseCommand):
    help = "Import the EU's Licences vocabulary into the database."

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        if not os.path.isfile("licences.xml"):
            self.stdout.write(
                self.style.ERROR(
                    "licences.xml does not exist. Please download it from https://op.europa.eu/s/y52h."
                )
            )
            return

        tree = ET.parse("licences.xml")
        root = tree.getroot()
        for record in root:
            if record.attrib["deprecated"] == "true":
                self.stdout.write(
                    self.style.WARNING(f"{licence.code} is deprecated. Skipping.")
                )
                continue
            code = record.find("authority-code").text
            label = [
                label.text
                for label in record.find("label").findall("lg.version")
                if label.attrib["lg"] == "eng"
            ][0]
            try:
                url_general = record.find("url.general").text
            except:
                url_general = None
                self.stdout.write(
                    self.style.ERROR(
                        f"{code} does not have a url.general. Setting it to None."
                    )
                )
            try:
                url_document = record.find("url.document").text
            except:
                url_document = None
                self.stdout.write(
                    self.style.ERROR(
                        f"{code} does not have a url.document. Setting it to None."
                    )
                )
            licence, created = LicenceDocument.objects.get_or_create(
                code=code,
                label=label,
                url_general=url_general,
                url_document=url_document,
            )
            if created:
                self.stdout.write(
                    self.style.SUCCESS(f"Successfully imported {licence.code}.")
                )
            else:
                self.stdout.write(
                    self.style.WARNING(f"{licence.code} already exists in the database.")
                )

        total_licences = LicenceDocument.objects.count()
        self.stdout.write(
            self.style.SUCCESS(f"Successfully imported {total_licences} licences.")
        )



# Generated by Django 5.0 on 2023-12-21 22:30

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0006_alter_datatheme_code"),
    ]

    operations = [
        migrations.AddField(
            model_name="dataset",
            name="issued",
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="dataset",
            name="modified",
            field=models.DateField(blank=True, null=True),
        ),
    ]



# Generated by Django 5.0 on 2024-01-05 16:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0009_alter_agent_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="dataset",
            name="landing_page",
            field=models.URLField(blank=True),
        ),
    ]



# Generated by Django 5.0 on 2023-12-21 22:39

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0007_dataset_issued_dataset_modified"),
    ]

    operations = [
        migrations.CreateModel(
            name="DataService",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("endpoint_url", models.URLField()),
                ("description", models.TextField(blank=True)),
                (
                    "catalog",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="dcat.catalog"
                    ),
                ),
                ("format", models.ManyToManyField(blank=True, to="dcat.mediatype")),
                (
                    "licence",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="dcat.licencedocument",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 5.0 on 2023-12-21 00:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0004_alter_catalog_themes_alter_dataset_description_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Keyword",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50)),
                ("slug", models.SlugField()),
            ],
        ),
        migrations.AddField(
            model_name="dataset",
            name="keywords",
            field=models.ManyToManyField(blank=True, to="dcat.keyword"),
        ),
    ]



import os

SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    'dcat',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.environ.get('DB_NAME', 'db.sqlite3'),
    }
}



# Generated by Django 5.0 on 2024-01-05 16:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0011_alter_distribution_file"),
    ]

    operations = [
        migrations.AddField(
            model_name="distribution",
            name="external_access_url",
            field=models.URLField(blank=True, default=""),
        ),
    ]



import json
import logging
import multiprocessing
import os

from django.core.management.base import BaseCommand

logging.basicConfig(filename="download-logs.txt", level=logging.INFO, format="")


def _download_resources_from_dataset(dataset):
    """Download the resources of a dataset.

    Resources will be downloaded in: `data/<dataset_id>/<resource_id>`
    If the folder already exists, it will be assumed as already donwloaded and the process will
    continue with the next one.
    """
    folder_name = "data"
    try:
        os.mkdir(f"{folder_name}/{dataset['identifier']}")
    except FileExistsError:
        pass

    resources = dataset["distribution"]

    logging.info(
        f"Downloading {len(resources)} resources from dataset {dataset['title']}"
    )

    for resource in resources:
        dir_name = f"{folder_name}/{dataset['identifier']}/{resource['identifier']}"
        try:
            os.mkdir(dir_name)
        except FileExistsError:
            # If the resource directory already exists, asume it has been downloaded
            # and continue with the next resource
            continue
        url = resource.get("downloadURL")

        if url is None:
            logging.error(f"Resource {resource['title']} does not have a download URL")
            continue

        exit_value = os.system(
            f"wget --connect-timeout 3 -t 2 --no-check-certificate --directory-prefix '{dir_name}' -q {url}"
        )
        if exit_value != 0:
            logging.error(f"Error when downloading: {url}")
            continue


class Command(BaseCommand):
    help = "Reads a data.json (ckanext-datajson) and downloads all files."

    def handle(self, *args, **options):
        data = {}
        with open("data.json", "r") as f:
            data = json.load(f)
        datasets = data["dataset"]

        try:
            os.mkdir("data")
        except FileExistsError:
            pass

        with multiprocessing.Pool(processes=8) as pool:
            pool.map(_download_resources_from_dataset, datasets)

        logging.info("Finished downloading all resources")



# Generated by Django 5.0 on 2024-01-05 16:33

import dcat.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0010_dataset_landing_page"),
    ]

    operations = [
        migrations.AlterField(
            model_name="distribution",
            name="file",
            field=models.FileField(
                blank=True, upload_to=dcat.models.Distribution._get_storage_path
            ),
        ),
    ]



""" Models for the DCAT application.

The models are based on the DCAT specification:
https://www.w3.org/TR/vocab-dcat-3/

Initial Mandatory/Recommended/Optional properties are based on the DCAT-AP
specification: https://semiceu.github.io/DCAT-AP/releases/3.0.0/

For the help_text we are copying the description of the properties from
the DCAT-AP specification.

"""
import hashlib

from django.db import models


class Agent(models.Model):
    """Any entity carrying out actions with respect to the (Core) entities.

    Example: Publishers, Creators, etc.
    """

    # Mandatory properties
    name = models.CharField(max_length=255, help_text="A name of the Agent.")

    # Recommended properties
    type = models.CharField(
        max_length=20,
        blank=True,
        help_text="A type of the agent that makes the Catalogue or Dataset available.",
    )

    # Optional properties
    # TODO: mbox is not defined in DCAT
    mbox = models.EmailField(
        blank=True, null=True, help_text="An email address of the Agent."
    )

    def to_jsonld(self):
        result = dict()
        result["foaf:name"] = self.name
        if self.type:
            result["@type"] = self.type
        return result

    def __str__(self):
        return self.name


class Catalog(models.Model):
    """A catalogue that hosts the Datasets or Data Services being described."""

    # Mandatory properties
    title = models.CharField(max_length=255, help_text="A name given to the Catalogue.")
    description = models.TextField(help_text="A free-text account of the Catalogue.")
    publisher = models.ForeignKey(
        "Agent",
        on_delete=models.CASCADE,
        help_text="An entity (organisation) responsible for making the Catalogue available.",
    )

    # Recommended properties
    licence = models.ForeignKey(
        "LicenceDocument",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        help_text="A licence under which the Catalogue can be used or reused.",
    )
    themes = models.ManyToManyField(
        "DataTheme",
        blank=True,
        help_text="A knowledge organization system used to classify the Catalogue's Datasets.",
    )
    homepage = models.URLField(
        blank=True, help_text="A web page that acts as the main page for the Catalogue."
    )

    def to_jsonld(self):
        result = dict()
        result["@type"] = "dcat:Catalog"
        result["dct:title"] = self.title
        result["dct:description"] = self.description
        result["dct:publisher"] = self.publisher.to_jsonld()
        if self.homepage:
            result["foaf:homepage"] = {"@type": "foaf:Document", "foaf:Document": self.homepage}

        result["dcat:dataset"] = [dataset.to_jsonld() for dataset in self.dataset_set.all()]

        return result

    def __str__(self):
        return self.title


# class CatalogRecord(models.Model):
#     """A description of a Dataset's entry in the Catalogue.

#     It exists for catalogs where a distinction is made between metadata about a dataset or
#     service and metadata about the entry in the catalog about the dataset or
#     service. For example, the publication date property of the dataset reflects
#     the date when the information was originally made available by the
#     publishing agency, while the publication date of the catalog record is
#     the date when the dataset was added to the catalog.
#     """
#     pass


class Dataset(models.Model):
    """A conceptual entity that represents the information published."""

    # Mandatory properties
    title = models.CharField(max_length=255, help_text="A name given to the Dataset.")
    catalog = models.ForeignKey("Catalog", on_delete=models.CASCADE)

    # Recommended properties
    # TODO: description is mandatory
    description = models.TextField(
        blank=True, help_text="A free-text account of the Dataset."
    )
    publisher = models.ForeignKey(
        "Agent",
        on_delete=models.SET_NULL,
        null=True,
        help_text="An entity (organisation) responsible for making the Dataset available.",
    )
    themes = models.ManyToManyField(
        "DataTheme",
        blank=True,
        help_text="A category of the Dataset. A Dataset may be associated with multiple themes.",
    )
    keywords = models.ManyToManyField(
        "Keyword", blank=True, help_text="A keyword or tag describing the Dataset."
    )

    # Optional properties
    modified = models.DateField(
        blank=True,
        null=True,
        help_text="The most recent date on which the Dataset was changed or modified.",
    )
    issued = models.DateField(blank=True, null=True)
    landing_page = models.URLField(
        blank=True,
        help_text="A web page that provides access to the Dataset, its Distributions and/or additional information. It is intended to point to a landing page at the original data provider, not to a page on a site of a third party, such as an aggregator.",
    )

    def to_jsonld(self):
        result = dict()
        result["dct:title"] = self.title
        if self.description:
            result["dct:description"] = self.description
        result["dcat:distribution"] = [d.to_jsonld() for d in self.distribution_set.all()]

        return result

    def __str__(self):
        return self.title


# class DatasetInSeries(models.Model):
#     """Capture the case when a Dataset is a member of a Dataset Series."""
#     pass


# class DatasetSeries(models.Model):
#     """Datasets that are published separately, but share some characteristics."""
#     pass


class Distribution(models.Model):
    """A physical embodiment of the Dataset in a particular format.

    Example: A CSV file, an RDF file, etc.
    """

    def _get_storage_path(instance, filename):
        """Return the storage path of the file.

        The OS can complain if we store thousands of files in
        the same directory. So I'm inventing something to avoid
        this problem.
        """
        return f"files/datasets/{instance.dataset.pk}/{filename}"

    # Mandatory properties
    dataset = models.ForeignKey("Dataset", on_delete=models.CASCADE)

    @property
    def access_url(self):
        """Return the access url of the file.

        If the file is hosted in another portal, the access_url is provided
        in the distribution. Otherwise, the access_url is the URL to the
        distribution.
        """
        if self.external_access_url:
            return self.external_access_url
        # TODO: Implement views in django-dcat
        return ""

    # Recomened properties
    title = models.CharField(
        max_length=255, blank=True, help_text="A name given to the Distribution."
    )
    description = models.TextField(
        blank=True, help_text="A free-text account of the Distribution."
    )
    file = models.FileField(upload_to=_get_storage_path, blank=True)
    format = models.ForeignKey(
        "MediaType",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        help_text="The file format of the Distribution.",
    )
    licence = models.ForeignKey(
        "LicenceDocument",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        help_text="A licence under which the Distribution is made available. ",
    )

    external_download_url = models.URLField(
        blank=True,
        default="",
        help_text="A URL that is a direct link to a downloadable file in a given format.",
    )
    external_access_url = models.URLField(
        blank=True,
        default="",
        help_text="A URL that gives access to a Distribution of the Dataset. The resource at the access URL may contain information about how to get the Dataset.",
    )

    # Optional properties
    checksum = models.OneToOneField(
        "Checksum",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        help_text="A mechanism that can be used to verify that the contents of a distribution have not changed. The checksum is related to the download_url.",
    )

    @property
    def download_url(self):
        """Return the download url of the file.

        If the file is hosted in another portal, the download_url is provided
        in the distribution. Otherwise, the download_url is generated from the
        file field.

        This field is not mandatory so it can return an empty string (a distrubution
        can contain only an access_url.)
        """
        if self.external_download_url:
            return self.external_download_url
        if self.file:
            return self.file.url
        return ""

    def calculate_md5_checksum(self):
        """Calculates the md5 checksum of the file."""
        md5_hash = hashlib.md5()
        with self.file.open(mode="rb") as f:
            while chunk := f.read(4096):
                md5_hash.update(chunk)
        return md5_hash.hexdigest()

    def to_jsonld(self):
        result = dict()
        result["@type"] = "dcat:Distribution"
        result["dcat:accessURL"] = self.access_url
        if self.title:
            result["dct:title"] = self.title
        if self.description:
            result["dct:description"] = self.description
        return result

    def __str__(self):
        return self.title


class DataService(models.Model):
    """Operations that provides access to datasets or data processing functions.

    Example: APIs, Web Services, SPARQL endpoints, etc.
    """

    # Mandatory properties
    title = models.CharField(max_length=255, help_text="A name given to the DataService.")
    endpoint_url = models.URLField(
        help_text="The root location or primary endpoint of the service (an IRI)."
    )
    catalog = models.ForeignKey("Catalog", on_delete=models.CASCADE)

    # Optional properties
    description = models.TextField(
        blank=True, help_text="A free-text account of the DataService."
    )
    format = models.ManyToManyField(
        "MediaType",
        blank=True,
        help_text="The structure that can be returned by querying the endpointURL.",
    )
    licence = models.ForeignKey(
        "LicenceDocument",
        on_delete=models.SET_NULL,
        null=True,
        help_text="A licence under which the Data service is made available.",
    )


class MediaType(models.Model):
    """A set of media types from the DCAT-AP vocabulary."""

    extension = models.CharField(max_length=10)
    code = models.CharField(max_length=10, unique=True, blank=True, null=True)
    media_type = models.CharField(max_length=50, blank=True)
    description = models.TextField(blank=True)

    def __str__(self):
        return self.extension


class LicenceDocument(models.Model):
    """A set of licences from the DCAT-AP vocabulary."""

    # Recommended properties
    @property
    def type(self):
        return self.url_general or self.label

    label = models.CharField(max_length=255)
    code = models.CharField(max_length=10, unique=True, blank=True, null=True)
    url_general = models.URLField(blank=True, default="")
    url_document = models.URLField(blank=True, default="")

    def __str__(self):
        return self.label


class DataTheme(models.Model):
    """Themes used for dataset classification.

    This is the mandatory controlled vocabulary for
    the themeTaxonomy field of Catalogue. (As defined
    by DCAT-AP).

    https://op.europa.eu/s/y52L
    """

    code = models.CharField(max_length=255, unique=True)
    label = models.CharField(max_length=255)
    description = models.TextField(blank=True, default="")

    def __str__(self):
        return self.label


class Checksum(models.Model):
    """A value that allows the contents of a file to be authenticated."""

    checksum_value = models.CharField(max_length=255)
    algorithm = models.CharField(max_length=10)

    def __str__(self):
        return f"{self.checksum_value} ({self.algorithm})"


class Keyword(models.Model):
    """A keyword or tag describing the Dataset."""

    name = models.CharField(max_length=50)
    slug = models.SlugField()

    def __str__(self):
        return self.name



# Generated by Django 5.0 on 2024-01-07 14:59

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0013_alter_catalog_licence"),
    ]

    operations = [
        migrations.AlterField(
            model_name="agent",
            name="mbox",
            field=models.EmailField(
                blank=True,
                help_text="An email address of the Agent.",
                max_length=254,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="agent",
            name="name",
            field=models.CharField(help_text="A name of the Agent.", max_length=255),
        ),
        migrations.AlterField(
            model_name="agent",
            name="type",
            field=models.CharField(
                blank=True,
                help_text="A type of the agent that makes the Catalogue or Dataset available.",
                max_length=20,
            ),
        ),
        migrations.AlterField(
            model_name="catalog",
            name="description",
            field=models.TextField(help_text="A free-text account of the Catalogue."),
        ),
        migrations.AlterField(
            model_name="catalog",
            name="homepage",
            field=models.URLField(
                blank=True,
                help_text="A web page that acts as the main page for the Catalogue.",
            ),
        ),
        migrations.AlterField(
            model_name="catalog",
            name="licence",
            field=models.ForeignKey(
                blank=True,
                help_text="A licence under which the Catalogue can be used or reused.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.licencedocument",
            ),
        ),
        migrations.AlterField(
            model_name="catalog",
            name="publisher",
            field=models.ForeignKey(
                help_text="An entity (organisation) responsible for making the Catalogue available.",
                on_delete=django.db.models.deletion.CASCADE,
                to="dcat.agent",
            ),
        ),
        migrations.AlterField(
            model_name="catalog",
            name="themes",
            field=models.ManyToManyField(
                blank=True,
                help_text="A knowledge organization system used to classify the Catalogue's Datasets.",
                to="dcat.datatheme",
            ),
        ),
        migrations.AlterField(
            model_name="catalog",
            name="title",
            field=models.CharField(
                help_text="A name given to the Catalogue.", max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="dataservice",
            name="description",
            field=models.TextField(
                blank=True, help_text="A free-text account of the DataService."
            ),
        ),
        migrations.AlterField(
            model_name="dataservice",
            name="endpoint_url",
            field=models.URLField(
                help_text="The root location or primary endpoint of the service (an IRI)."
            ),
        ),
        migrations.AlterField(
            model_name="dataservice",
            name="format",
            field=models.ManyToManyField(
                blank=True,
                help_text="The structure that can be returned by querying the endpointURL.",
                to="dcat.mediatype",
            ),
        ),
        migrations.AlterField(
            model_name="dataservice",
            name="licence",
            field=models.ForeignKey(
                help_text="A licence under which the Data service is made available.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.licencedocument",
            ),
        ),
        migrations.AlterField(
            model_name="dataservice",
            name="title",
            field=models.CharField(
                help_text="A name given to the DataService.", max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="description",
            field=models.TextField(
                blank=True, help_text="A free-text account of the Dataset."
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="keywords",
            field=models.ManyToManyField(
                blank=True,
                help_text="A keyword or tag describing the Dataset.",
                to="dcat.keyword",
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="landing_page",
            field=models.URLField(
                blank=True,
                help_text="A web page that provides access to the Dataset, its Distributions and/or additional information. It is intended to point to a landing page at the original data provider, not to a page on a site of a third party, such as an aggregator.",
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="modified",
            field=models.DateField(
                blank=True,
                help_text="The most recent date on which the Dataset was changed or modified.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="publisher",
            field=models.ForeignKey(
                help_text="An entity (organisation) responsible for making the Dataset available.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.agent",
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="themes",
            field=models.ManyToManyField(
                blank=True,
                help_text="A category of the Dataset. A Dataset may be associated with multiple themes.",
                to="dcat.datatheme",
            ),
        ),
        migrations.AlterField(
            model_name="dataset",
            name="title",
            field=models.CharField(
                help_text="A name given to the Dataset.", max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="checksum",
            field=models.OneToOneField(
                blank=True,
                help_text="A mechanism that can be used to verify that the contents of a distribution have not changed. The checksum is related to the download_url.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.checksum",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="description",
            field=models.TextField(
                blank=True, help_text="A free-text account of the Distribution."
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="external_access_url",
            field=models.URLField(
                blank=True,
                default="",
                help_text="A URL that gives access to a Distribution of the Dataset. The resource at the access URL may contain information about how to get the Dataset.",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="external_download_url",
            field=models.URLField(
                blank=True,
                default="",
                help_text="A URL that is a direct link to a downloadable file in a given format.",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="format",
            field=models.ForeignKey(
                blank=True,
                help_text="The file format of the Distribution.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.mediatype",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="licence",
            field=models.ForeignKey(
                blank=True,
                help_text="A licence under which the Distribution is made available. ",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.licencedocument",
            ),
        ),
        migrations.AlterField(
            model_name="distribution",
            name="title",
            field=models.CharField(
                blank=True,
                help_text="A name given to the Distribution.",
                max_length=255,
            ),
        ),
    ]



from django.test import TestCase
from dcat.models import Agent, Catalog, Dataset, Distribution


class DCATSerializationJSONLDTestCase(TestCase):
    """ Test that Catalog is properly serialized to JSONLD format.

    This TestCase is inspired by https://data-in-emergencies.fao.org/api/feed/dcat-ap/2.1.1.json.
    """
    def setUp(self):
        publisher = Agent.objects.create(
            name='Food and Agriculture Organization of the United Nations',
            type='foaf:Agent'
        )
        catalog = Catalog.objects.create(
            title='FAO Data in Emergencies',
            description='A testing catalog based on true data.',
            publisher=publisher,
            homepage='https://data-in-emergencies.fao.org'
        )

        dataset = Dataset.objects.create(
            title='Colombia - Household Questionnaire - Round 3',
            description='Household questionarie',
            catalog=catalog,
        )
        Dataset.objects.create(
            title='Afghanistan - Household Questionnaire - Round 6',
            catalog=catalog,
        )

        Distribution.objects.create(
            dataset=dataset,
            title='ArcGIS Hub Dataset',
            description='Web page',
            external_access_url='https://external.com/distribution/webpage'
        )

        Distribution.objects.create(
            dataset=dataset,
            title='ArcGIS GeoService',
            description='Esri REST',
            external_access_url='https://external.com/distribution/rest'
        )

    def test_distribution_to_jsonld(self):
        distribution = Distribution.objects.first()
        result = distribution.to_jsonld()
        self.assertEqual(result['@type'], 'dcat:Distribution')
        self.assertEqual(result['dcat:accessURL'], distribution.access_url)
        self.assertEqual(result['dct:title'], distribution.title)
        self.assertEqual(result['dct:description'], distribution.description)

    def test_agent_to_jsonld(self):
        publisher = Agent.objects.first()
        result = publisher.to_jsonld()
        self.assertEqual(result['@type'], publisher.type)
        self.assertEqual(result['foaf:name'], publisher.name)

    def test_dataset_to_jsonld(self):
        dataset = Dataset.objects.first()
        result = dataset.to_jsonld()
        self.assertEqual(result['dct:title'], 'Colombia - Household Questionnaire - Round 3')
        self.assertEqual(result['dct:description'], 'Household questionarie')

        d0, d1 = Distribution.objects.all()
        self.assertEqual(result['dcat:distribution'][0], d0.to_jsonld())
        self.assertEqual(result['dcat:distribution'][1], d1.to_jsonld())

    def test_catalog_to_jsonld(self):
        catalog = Catalog.objects.first()
        result = catalog.to_jsonld()

        self.assertEqual(result['@type'], 'dcat:Catalog')
        self.assertEqual(result['dct:title'], catalog.title)
        self.assertEqual(result['dct:description'], catalog.description)
        self.assertEqual(result['dct:publisher'], catalog.publisher.to_jsonld())
        self.assertEqual(result['foaf:homepage'], {'@type': 'foaf:Document', 'foaf:Document': catalog.homepage})

        dataset = Dataset.objects.first()
        self.assertEqual(result['dcat:dataset'][0], dataset.to_jsonld())

        d0, d1 = Distribution.objects.all()
        self.assertEqual(result['dcat:dataset'][0]['dcat:distribution'][0], d0.to_jsonld())
        self.assertEqual(result['dcat:dataset'][0]['dcat:distribution'][1], d1.to_jsonld())



# Generated by Django 5.0 on 2024-01-06 20:55

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0012_distribution_external_access_url"),
    ]

    operations = [
        migrations.AlterField(
            model_name="catalog",
            name="licence",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dcat.licencedocument",
            ),
        ),
    ]



# Generated by Django 5.0 on 2023-12-20 22:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("dcat", "0002_rename_license_catalog_licence_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="catalog",
            name="homepage",
            field=models.URLField(blank=True),
        ),
    ]
